//модуль для клиента
'use strict';
// это клиент

// Зависимости
let app = require('express')();
let timestamp = require('time-stamp');
const logSymbols = require('log-symbols');
let _ = require('lodash');
let ip = require('ip');
let nanoid = require('nanoid');
let axios = require('axios');

// Конфигурация
const wsName = require('../../package.json').name;
const wsVersion = require('../../package.json').version;
const wsClientConfig = require('../../wsClientConfig.json');
const wsCoreKey = wsClientConfig.wsCoreKey;
const wsPort = wsClientConfig.wsPort;
const wsEventBusAddress = wsClientConfig.wsEventBusAddress;

const instanceID = nanoid(10);

// Характеристики инстанса в сессии сокет-соединения
let socket = require('socket.io-client')(wsEventBusAddress,{
    query: {
        token: wsCoreKey, //jwt
        wsName: wsName,
        instanceID: instanceID
    }
});

// Характеристики инстанса отправляемые сервисам ядра
let wsStartData = {
    instanceID: instanceID,
    instanceVer: wsVersion,
    instanceIP: ip.address(),
    instancePort: wsPort,
    wsName: wsName
};

// Отправляем данные о запущенном сервисе
// При успешном коннекте
socket.on('connect', function() {
    socket.emit('wsStart', wsStartData);
});

// При успешном коннекте
socket.on('afterCoreRestart', function(data) {
    console.log(data);
    socket.emit('wsStart', wsStartData);
});

socket.on('coreIsDisabled', function(data) {
    console.log(data);
});


socket.on('regiseredInSystem', function(data) {
    console.log(data);
});

socket.on('error', (error) => {
    console.log(error)
});


let msSocket = {
    on: function (eventName, onEventAction) {
        let personalEventName = `${wsStartData.instanceID}/${eventName}`;
        //console.log(personalEventName);
        function step1 (data){
            //console.log(`Получено сообщение с идентификатором: ${data.eventId}`);
            let dataToChangeEventState = {'eventId':data.eventId,'eventState':'received'};
            socket.emit('changeEventState', dataToChangeEventState);
        }
        async function step2 (data){
            try {
                await onEventAction(data);
                step3(data);
            } catch (err) {
                //console.log(`Ошибка обработки по сообщению с идентификатором: ${data.eventId}`);
                let dataToChangeEventState = {'eventId':data.eventId,'eventState':'error'};
                socket.emit('changeEventState', dataToChangeEventState);
            }
        }
        function step3 (data){
            //console.log(`Выполнена обработка по сообщению с идентификатором: ${data.eventId}`);
            let dataToChangeEventState = {'eventId':data.eventId,'eventState':'succsess'};
            socket.emit('changeEventState', dataToChangeEventState);
        }
        function run (data){
            step1(data);
            step2(data);
        }
        socket.on(personalEventName, function(data){
            run(data)
        });
        // Регистрация подписки в MQ всех обработчиков, принадлежащих сервису
        socket.emit('regMeOnMQ', {wsName:wsStartData.wsName, instanceID:wsStartData.instanceID, eventName:eventName, personalEventName: personalEventName});
    },
    emit: function (eventName, data) {
        socket.emit('universalEvent', {'eventName':eventName, 'data':data});
    }
};

async function callAxiGet(axiosPath) {
    const axiosRes = await axios.get(axiosPath);
    return axiosRes.data
}

// Проверка что запрос пришел именно через apigw
app.use(function(req, res, next){
    if (req.method ==='OPTIONS'){
        next();
    }
    else {
        //console.log(`Промежуточная функция до вызова метода`);
        if (req.method ==='GET' && req.query.instanceID === instanceID || (req.method ==='POST' && req.body.instanceID === instanceID)){
            next();
        } else {
            return res.send({result: 'error', reason: 'Запрос к сервису минуя apigw'})
        }

    }
});

app.listen(wsPort, function(){
    console.log(`${timestamp('[DD-MM-YYYY HH:mm:ss.ms]')} ${logSymbols.success} Сервис ${wsName} запущен на ${wsPort} порту`)
});

module.exports = {
    callAxiGet: async function(axiPath, axiParams ) {
        let allParams = {fromWsName:wsName, fromInstanceID:instanceID};
        if (axiParams !== undefined) {
            Object.assign(axiParams, allParams);
        }
        const axiosRes = await axios.get(axiPath, {params:allParams});
        return axiosRes.data
    },
    callAxiPost: async function(axiosPath) {
        const axiosRes = await axios.post(axiosPath);
        return axiosRes.data
    },
    msSocket: msSocket,
    app: app
};

